import argparse
import json
from urllib.parse import urlparse

from .sslclient import SSLClient
from .view import print_ssl_info


class CommandArgs:
    def __init__(self):
        parser = argparse.ArgumentParser(description="Process some integers.")
        parser.add_argument(
            "-k", help="Do not validate certificate", action="store_true"
        )
        parser.add_argument("-v", help="Set verbose mode", action="store_true")
        parser.add_argument(
            "-f", help="Select field to print", nargs="+", type=str, action="append"
        )
        parser.add_argument("hostname", help="Connect to hostname", metavar="HOSTNAME")
        parser.add_argument(
            "--port", help="Using port", metavar="PORT", type=int, default=443
        )
        parser.add_argument(
            "--timeout",
            help="Max connection timeout (secs)",
            metavar="TIMEOUT",
            type=int,
            default=10,
        )
        parser.add_argument("-j", help="Produce json output", action="store_true")
        self.parser = parser

    def parse(self):
        self.args = self.parser.parse_args()

    def run(self):

        hostname = self.args.hostname
        port = self.args.port
        if "//" in hostname:
            url_parts = urlparse(hostname)
            hostname = url_parts.netloc
        if ":" in hostname:
            hostname, port = hostname.split(":")
            port = int(port)

        ssl_client = SSLClient()
        ssl_client.connect(hostname, port, self.args.timeout, self.args.k is True)
        x509dict = ssl_client.x509dict(add_hints=not self.args.j)
        if self.args.f:
            for key in self.args.f[0]:
                print(x509dict[key])
                return
        if self.args.j:
            print(json.dumps(x509dict, indent=4))
        else:
            print_ssl_info(x509dict)



import ssl
import socket
import sys
import OpenSSL.crypto as crypto
from datetime import datetime


def asn1time(time_str):
    time_obj = datetime.strptime(time_str.decode("ascii"), "%Y%m%d%H%M%SZ")
    return time_obj.strftime("%c")


def expires_in(time_until: str):
    time_from = datetime.now()
    time_until = datetime.strptime(time_until.decode("ascii"), "%Y%m%d%H%M%SZ")
    delta = (time_until - time_from).days + 1
    return delta


def get_certificate_san(x509cert):
    san = ""
    ext_count = x509cert.get_extension_count()
    for i in range(0, ext_count):
        ext = x509cert.get_extension(i)
        if "subjectAltName" in str(ext.get_short_name()):
            san = ext.__str__()
    return san


class SSLClient:
    def connect(self, hostname, port, timeout=10, skip_ssl_verification=False):
        ctx = ssl.create_default_context()
        if skip_ssl_verification:
            ctx.check_hostname = False
            ctx.verify_mode = ssl.CERT_NONE
        with ctx.wrap_socket(socket.socket(), server_hostname=hostname) as s:
            print(f"* Connecting to {hostname}, port {port}", file=sys.stderr)
            s.settimeout(timeout)
            s.connect((hostname, port))
            cert_bin = s.getpeercert(True)
            x509 = crypto.load_certificate(crypto.FILETYPE_ASN1, cert_bin)
            self.x509 = x509
            self.hostname = hostname
            self.port = port

    def x509dict(self, add_hints: bool):
        info_dict = {}
        x509 = self.x509

        components = []
        for component in x509.get_subject().get_components():
            key, value = component
            components.append(f"{key.decode()}={value.decode()}")
        info_dict["Hostname"] = self.hostname
        info_dict["Port"] = self.port
        info_dict["subject"] = "; ".join(components)
        info_dict["subjectAltName"] = get_certificate_san(x509)
        info_dict["SHA256"] = x509.digest("SHA256").decode()
        info_dict["Serial"] = x509.get_serial_number()

        components = []
        for component in x509.get_issuer().get_components():
            key, value = component
            components.append(f"{key.decode()}={value.decode()}")
        info_dict["issuer"] = "; ".join(components)

        info_dict["Valid from"] = asn1time(x509.get_notBefore())
        info_dict["Valid until"] = asn1time(x509.get_notAfter())

        expires_in_days = expires_in(x509.get_notAfter())
        if add_hints:
            if x509.has_expired():  # Yes, inverted logic for testing :)
                info_dict[
                    "Valid until"
                ] = f'[bold][red]{info_dict["Valid until"]} (expired !!!)[/red][/bold]'
            else:
                info_dict[
                    "Valid until"
                ] = f'{info_dict["Valid until"]} - expires in [green]{expires_in_days} day(s)[/green]'

        info_dict["SigAlgorithm"] = x509.get_signature_algorithm().decode()
        info_dict["Version"] = x509.get_version()
        return info_dict



from rich.console import Console
from rich.table import Table


def print_ssl_info(ssl_info):
    console = Console()

    table = Table(show_header=False)
    for key, value in ssl_info.items():
        table.add_row(key, str(value))
    console.print(table)
